#pragma once 
namespace pco
{
  class Camera;
  class Image;
  using ImagePtr = std::shared_ptr<Image>;

  class Image
  {
  private:
    // used for 8 bit cameras and compressed images
    std::variant<std::monostate, image8_t, image16_t> raw_data_; // camera + roi settings
    std::variant<std::monostate, image8_t, image16_t> data_; // data_format: Mono8, BGR8, BGR16

    Roi roi_; // TODO: change default UNDEF to 0;
    DataFormat data_format_ = DataFormat::Undefined;
    DWORD recorder_image_number_ = 0; // average image has no number

  public:
    //_________________________________________
    // constructor + destructor

    explicit Image(); // allocation takes place in Camera::image funcs
    explicit Image(Roi roi, DataFormat data_format, RawFormat raw_format = RawFormat::Word); // pre-allocation in constructor
    ~Image();

    //_________________________________________
    // image properties

    bool is8Bit();
    bool is16Bit();
    bool isColored();
    DataFormat getDataFormat();

    //_________________________________________
    // allocation

    size_t width();
    size_t height();
    bool validAllocation(Roi roi, DataFormat data_format, RawFormat raw_format = RawFormat::Word);
    void resize(Roi roi, DataFormat data_format, RawFormat raw_format = RawFormat::Word);

    //_________________________________________
    // meta data

    void setRecorderImageNumber(DWORD recorder_image_number);
    DWORD getRecorderImageNumber();
    void setMetaData(PCO_METADATA_STRUCT _meta_data = PCO_METADATA_STRUCT{ sizeof(PCO_METADATA_STRUCT) });
    void clearMetaData();


    PCO_METADATA_STRUCT* getMetaDataPtr();
    PCO_METADATA_STRUCT getMetaData(); // copies


    void setTimestamp(PCO_TIMESTAMP_STRUCT _timestamp = PCO_TIMESTAMP_STRUCT{ sizeof(PCO_TIMESTAMP_STRUCT) });
    void clearTimestamp();
    PCO_TIMESTAMP_STRUCT* getTimestampPtr();

    std::optional<PCO_METADATA_STRUCT> meta_data; // direct access
    std::optional<PCO_TIMESTAMP_STRUCT> timestamp; // direct access

    //_________________________________________
    // data access

    std::pair<void*, size_t> raw_data(); // raw pointer to raw memory, currently: mono 16bit
    std::pair<void*, size_t> data(); // raw pointer to output vector memory, consider bitness and pixel size
    size_t size(); // raw size


    image8_t& vector_8bit();
    image16_t& vector_16bit();
    image16_t& raw_vector_16bit();

  };

};
